#include <stdio.h>
#include <iLBC_decode.h>
#include <iLBC_define.h>
#define ILBC_FIX_MODE 20
/*----------------------------------------------------------------*
 *  Decoder interface function
 *----------------------------------------------------------------*/

short decode( /* (o) Number of decoded samples */
iLBC_Dec_Inst_t *iLBCdec_inst, /* (i/o) Decoder instance */
short *decoded_data, /* (o) Decoded signal block*/
short *encoded_data, /* (i) Encoded bytes */
short mode /* (i) 0=PL, 1=Normal */
) {
	int k;
	float decblock[BLOCKL_MAX], dtmp;

	/* check if mode is valid */

	if (mode < 0 || mode > 1) {
		printf("\nERROR - Wrong mode - 0, 1 allowed\n");
		exit(3);
	}

	/* do actual decoding of block */

	iLBC_decode(decblock, (unsigned char *) encoded_data, iLBCdec_inst, mode);

	/* convert to short */
	for (k = 0; k < iLBCdec_inst->blockl; k++) {
		dtmp = decblock[k];

		if (dtmp < MIN_SAMPLE)
			dtmp = MIN_SAMPLE;
		else if (dtmp > MAX_SAMPLE)
			dtmp = MAX_SAMPLE;
		decoded_data[k] = (short) dtmp;
	}

	return (iLBCdec_inst->blockl);
}
int main(int argc, char *argv[]) {

	FILE *ifileid, *ofileid;
	short data[BLOCKL_MAX];
	if (argc != 3) {
		fprintf(stderr, "Wrong parameter: encode PCM_file_in Ilbc_file_out\n");
		exit(-1);
	}
	if ((ifileid = fopen(argv[1], "rb")) == NULL) {
		fprintf(stderr, "Cannot open input file %s\n", argv[2]);
		exit(2);
	}
	if ((ofileid = fopen(argv[2], "wb")) == NULL) {
		fprintf(stderr, "Cannot open encoded file %s\n", argv[3]);
		exit(1);
	}
	int len;
	printf("decode start\n");
	iLBC_Dec_Inst_t Dec_Inst;
	initDecode(&Dec_Inst, ILBC_FIX_MODE, 1);
	fseek(ifileid, 0, SEEK_END);
	int inlen = ftell(ifileid);
	int inpos = 0;
	char *input = (char*)malloc(inlen);
	fseek(ifileid, 0, SEEK_SET);
	fread(input, 1, inlen, ifileid);
	while (inpos < inlen) {
		len = decode(&Dec_Inst, data, &input[inpos],1);
		fwrite(data, sizeof(short), len, ofileid);
		inpos += Dec_Inst.no_of_bytes;
	}
	free(input);
	fclose(ifileid);
	fclose(ofileid);
	return 0;
}
